/************************************************************************
* Copyright (c) 1995 Valentin Shopov, Blagoevgrad, Bulgaria             *
*                                                                       *
* Permission to use, copy, modify, and distribute this material for any *
* purpose and without fee is hereby granted, provided that the above    *
* copyright notice and this permission notice appear in all copies, and *
* that the name of the author not be used in advertising or publicity   *
* pertaining to this material without the specific, prior written       *
* permission of the author.                                             *
*                                                                       *
* THE AUTHOR MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY *
* OF THIS MATERIAL FOR ANY PURPOSE. IT IS PROVIDED "AS IS", WITHOUT ANY *
* EXPRESS OR IMPLIED WARRANTIES.                                        *
*************************************************************************/

/**************************ciilib1.4.0***********************************
 *                                                                      *
 *        Character to Inline Image Conversion Library (CIILIB)         *
 *                                                                      *
 *         	  x16 and x14 CII Fonts Builder.			*
 *                      by Valentin Shopov                              *
 *                      valentin@rs.aubg.bg                             *
 ************************************************************************/


#include <stdio.h>
#include "cii.h"

#define A_7bit  "7bit"
#define A_8bit  "8bit"
#define A_1byte "1byte"
#define A_2byte "2byte"

extern struct CharCodesXBM CII_CC[64];

/* imported from ciiserv.c */
static char *
Sprintf (out, fmt, a, b, c, d, e, f, g)
     char *out, *fmt, *a, *b, *c, *d, *e, *f, *g;
{
  sprintf (out, fmt, a, b, c, d, e, f, g);
  return out + strlen (out);
}

/* imported from ciiserv.c */
CII_putFonts2 (csx, tc)
     struct CharCodesXBM *csx;
     FILE *tc;
{
  int leng;
  char head[2048], *hp;
  int map, mapx;
  char *bp;
  int nx, xi, cx, nb, bsize, bi;

  if (csx->name == 0)
    return 0;

  hp = head;
  hp = Sprintf (hp, "CII-Version: 0.1\n");
  hp = Sprintf (hp, "Name: %s,%s\n", csx->regname, csx->name);
  hp = Sprintf (hp, "Size: %d,%d\n", csx->width, csx->height);
  hp = Sprintf (hp, "Range: %d,%d\n", csx->first, csx->last);
  hp = Sprintf (hp, "Attrs: ");
  if (csx->is2bytes)
    hp = Sprintf (hp, A_2byte);
  else
    hp = Sprintf (hp, A_1byte);
  hp = Sprintf (hp, ",");
  if (csx->has8bit)
    hp = Sprintf (hp, A_8bit);
  else
    hp = Sprintf (hp, A_7bit);
  hp = Sprintf (hp, "\n");
  hp = Sprintf (hp, "\f\n\n");
  fputs (head, tc);
  leng = strlen (head);

  nx = csx->last - csx->first + 1;
  map = 0;
  mapx = 0;
  nb = 0;
  for (xi = 0; xi < nx; xi++)
    {
      if (0 <= csx->xbmind[xi])
	{
	  nb++;
	  map |= (1 << mapx);
	}
      if (++mapx == 8)
	{
	  putc (map, tc);
	  leng++;
	  map = 0;
	  mapx = 0;
	}
    }
  if (mapx != 0)
    {
      putc (map, tc);
      leng++;
    }
  bsize = (csx->width * csx->height) / 8;
  nb = 0;
  for (xi = 0; xi < nx; xi++)
    {
      if ((cx = csx->xbmind[xi]) < 0)
	continue;
      bp = csx->xbmsrc[cx];
      for (bi = 0; bi < bsize; bi++)
	putc (bp[bi], tc);
      nb++;
    }
  leng += nb * bsize;
  return leng;
}

main (ac, av)
     char *av[];
{
  int i;
  FILE *ciif;
  char outpf[2048];

  if (ac < 2)
    {
      fprintf (stderr, "Usage: %s  <CII Fonts Directory>\n", av[0]);
      exit (1);
    }


  for (i = 0; CII_CC[i].regname; i++)
    {
      sprintf (outpf, "%s/%s", av[1], (char *) CII_CC[i].regname);
      ciif = fopen (outpf, "w+");
      if (ciif == NULL)
	{
	  fprintf (stderr, "Error opening \"%s\"!\n", outpf);
	  exit (1);
	}
      CII_putFonts2 (&CII_CC[i], ciif);
      printf (" CIIFONT=\"file:%s\"", outpf);
      fclose (ciif);

    }
  printf ("\n");
  exit (0);

}
